@@ -1,6 +1,6 @@
 from typing import Optional
 
-from opentelemetry.semconv.ai import TraceloopSpanKindValues
+from opentelemetry.semconv_ai import TraceloopSpanKindValues
 
 from traceloop.sdk.decorators.base import (
     aentity_class,
@@ -80,9 +80,7 @@ def atask(
     if method_name is None:
         return aentity_method(name=name, version=version, tlp_span_kind=tlp_span_kind)
     else:
-        return aentity_class(
-            name=name, method_name=method_name, tlp_span_kind=tlp_span_kind
-        )
+        return aentity_class(name=name, version=version, method_name=method_name, tlp_span_kind=tlp_span_kind)
 
 
 def aworkflow(
