@@ -99,8 +99,11 @@ def fetch_url(url: str, api_key: str):
 
     if response.status_code != 200:
         if response.status_code == 401 or response.status_code == 403:
-            logging.error("Authorization error: Invalid API key.")
-        raise requests.exceptions.HTTPError(response=response)
+            logging.error("Authorization error: Invalid Traceloop API key.")
+            raise requests.exceptions.HTTPError(response=response)
+        else:
+            logging.error("Request failed: %s", response.status_code)
+            raise requests.exceptions.HTTPError(response=response)
     else:
         return response.json()
 
