@@ -2,26 +2,32 @@ from enum import Enum
 
 
 class Instruments(Enum):
-    OPENAI = "openai"
+    ALEPHALPHA = "alephalpha"
     ANTHROPIC = "anthropic"
-    COHERE = "cohere"
-    PINECONE = "pinecone"
+    BEDROCK = "bedrock"
     CHROMA = "chroma"
+    COHERE = "cohere"
     GOOGLE_GENERATIVEAI = "google_generativeai"
+    GROQ = "groq"
+    HAYSTACK = "haystack"
+    LANCEDB = "lancedb"
     LANGCHAIN = "langchain"
+    LLAMA_INDEX = "llama_index"
+    MARQO = "marqo"
+    MILVUS = "milvus"
     MISTRAL = "mistral"
     OLLAMA = "ollama"
-    LLAMA_INDEX = "llama_index"
-    MILVUS = "milvus"
-    TRANSFORMERS = "transformers"
-    TOGETHER = "together"
-    REQUESTS = "requests"
-    URLLIB3 = "urllib3"
+    OPENAI = "openai"
+    PINECONE = "pinecone"
     PYMYSQL = "pymysql"
-    BEDROCK = "bedrock"
+    QDRANT = "qdrant"
+    REDIS = "redis"
     REPLICATE = "replicate"
+    REQUESTS = "requests"
+    SAGEMAKER = "sagemaker"
+    TOGETHER = "together"
+    TRANSFORMERS = "transformers"
+    URLLIB3 = "urllib3"
     VERTEXAI = "vertexai"
     WATSONX = "watsonx"
     WEAVIATE = "weaviate"
-    ALEPHALPHA = "alephalpha"
-    MARQO = "marqo"
