@@ -22,14 +22,16 @@ from opentelemetry.sdk.trace.export import (
 )
 from opentelemetry.trace import get_tracer_provider, ProxyTracerProvider
 from opentelemetry.context import get_value, attach, set_value
+from opentelemetry.instrumentation.threading import ThreadingInstrumentor
 
-from opentelemetry.semconv.ai import SpanAttributes
+from opentelemetry.semconv_ai import SpanAttributes
 from traceloop.sdk import Telemetry
+from traceloop.sdk.images.image_uploader import ImageUploader
 from traceloop.sdk.instruments import Instruments
 from traceloop.sdk.tracing.content_allow_list import ContentAllowList
 from traceloop.sdk.utils import is_notebook
 from traceloop.sdk.utils.package_check import is_package_installed
-from typing import Dict, Optional, Set
+from typing import Callable, Dict, Optional, Set
 
 
 TRACER_NAME = "traceloop.tracer"
@@ -46,6 +48,7 @@ EXCLUDED_URLS = """
     posthog.com,
     sentry.io,
     bedrock-runtime,
+    sagemaker-runtime,
     googleapis.com,
     githubusercontent.com,
     openaipublic.blob.core.windows.net"""
@@ -57,6 +60,7 @@ class TracerWrapper(object):
     endpoint: str = None
     headers: Dict[str, str] = {}
     __tracer_provider: TracerProvider = None
+    __image_uploader: ImageUploader = None
 
     def __new__(
         cls,
@@ -66,12 +70,15 @@ class TracerWrapper(object):
         exporter: SpanExporter = None,
         should_enrich_metrics: bool = True,
         instruments: Optional[Set[Instruments]] = None,
+        block_instruments: Optional[Set[Instruments]] = None,
+        image_uploader: ImageUploader = None,
     ) -> "TracerWrapper":
         if not hasattr(cls, "instance"):
             obj = cls.instance = super(TracerWrapper, cls).__new__(cls)
             if not TracerWrapper.endpoint:
                 return obj
 
+            obj.__image_uploader = image_uploader
             obj.__resource = Resource(attributes=TracerWrapper.resource_attributes)
             obj.__tracer_provider = init_tracer_provider(resource=obj.__resource)
             if processor:
@@ -119,193 +126,15 @@ class TracerWrapper(object):
             if propagator:
                 set_global_textmap(propagator)
 
-            instrument_set = False
-            if instruments is None:
-                init_instrumentations(should_enrich_metrics)
-                instrument_set = True
-            else:
-                for instrument in instruments:
-                    if instrument == Instruments.OPENAI:
-                        if not init_openai_instrumentor(should_enrich_metrics):
-                            print(Fore.RED + "Warning: OpenAI library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.ANTHROPIC:
-                        if not init_anthropic_instrumentor(should_enrich_metrics):
-                            print(
-                                Fore.RED + "Warning: Anthropic library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.COHERE:
-                        if not init_cohere_instrumentor():
-                            print(Fore.RED + "Warning: Cohere library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.PINECONE:
-                        if not init_pinecone_instrumentor():
-                            print(
-                                Fore.RED + "Warning: Pinecone library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.CHROMA:
-                        if not init_chroma_instrumentor():
-                            print(Fore.RED + "Warning: Chroma library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.GOOGLE_GENERATIVEAI:
-                        if not init_google_generativeai_instrumentor():
-                            print(
-                                Fore.RED
-                                + "Warning: Google Generative AI library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.LANGCHAIN:
-                        if not init_langchain_instrumentor():
-                            print(
-                                Fore.RED + "Warning: LangChain library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.MISTRAL:
-                        if not init_mistralai_instrumentor():
-                            print(
-                                Fore.RED + "Warning: MistralAI library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.OLLAMA:
-                        if not init_ollama_instrumentor():
-                            print(Fore.RED + "Warning: Ollama library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.LLAMA_INDEX:
-                        if not init_llama_index_instrumentor():
-                            print(
-                                Fore.RED + "Warning: LlamaIndex library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.MILVUS:
-                        if not init_milvus_instrumentor():
-                            print(Fore.RED + "Warning: Milvus library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.TRANSFORMERS:
-                        if not init_transformers_instrumentor():
-                            print(
-                                Fore.RED
-                                + "Warning: Transformers library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.TOGETHER:
-                        if not init_together_instrumentor():
-                            print(
-                                Fore.RED + "Warning: TogetherAI library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.REQUESTS:
-                        if not init_requests_instrumentor():
-                            print(
-                                Fore.RED + "Warning: Requests library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.URLLIB3:
-                        if not init_urllib3_instrumentor():
-                            print(Fore.RED + "Warning: urllib3 library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.PYMYSQL:
-                        if not init_pymysql_instrumentor():
-                            print(Fore.RED + "Warning: PyMySQL library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.BEDROCK:
-                        if not init_bedrock_instrumentor(should_enrich_metrics):
-                            print(Fore.RED + "Warning: Bedrock library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.REPLICATE:
-                        if not init_replicate_instrumentor():
-                            print(
-                                Fore.RED + "Warning: Replicate library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.VERTEXAI:
-                        if not init_vertexai_instrumentor():
-                            print(
-                                Fore.RED + "Warning: Vertex AI library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.WATSONX:
-                        if not init_watsonx_instrumentor():
-                            print(Fore.RED + "Warning: Watsonx library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.WEAVIATE:
-                        if not init_weaviate_instrumentor():
-                            print(
-                                Fore.RED + "Warning: Weaviate library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.ALEPHALPHA:
-                        if not init_alephalpha_instrumentor():
-                            print(
-                                Fore.RED
-                                + "Warning: Aleph Alpha library does not exist."
-                            )
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
-                    elif instrument == Instruments.MARQO:
-                        if not init_marqo_instrumentor():
-                            print(Fore.RED + "Warning: marqo library does not exist.")
-                            print(Fore.RESET)
-                        else:
-                            instrument_set = True
+            # this makes sure otel context is propagated so we always want it
+            ThreadingInstrumentor().instrument()
 
-                    else:
-                        print(
-                            Fore.RED
-                            + "Warning: "
-                            + instrument
-                            + " instrumentation does not exist."
-                        )
-                        print(
-                            "Usage:\n"
-                            + "from traceloop.sdk.instruments import Instruments\n"
-                            + 'Traceloop.init(app_name="...", instruments=set([Instruments.OPENAI]))'
-                        )
-                        print(Fore.RESET)
+            instrument_set = init_instrumentations(
+                should_enrich_metrics,
+                image_uploader.aupload_base64_image,
+                instruments,
+                block_instruments,
+            )
 
             if not instrument_set:
                 print(
@@ -329,9 +158,9 @@ class TracerWrapper(object):
         if workflow_name is not None:
             span.set_attribute(SpanAttributes.TRACELOOP_WORKFLOW_NAME, workflow_name)
 
-        entity_name = get_value("entity_name")
-        if entity_name is not None:
-            span.set_attribute(SpanAttributes.TRACELOOP_ENTITY_NAME, entity_name)
+        entity_path = get_value("entity_path")
+        if entity_path is not None:
+            span.set_attribute(SpanAttributes.TRACELOOP_ENTITY_PATH, entity_path)
 
         association_properties = get_value("association_properties")
         if association_properties is not None:
@@ -444,12 +273,12 @@ def set_workflow_name(workflow_name: str) -> None:
     attach(set_value("workflow_name", workflow_name))
 
 
-def set_entity_name(entity_name: str) -> None:
-    attach(set_value("entity_name", entity_name))
+def set_entity_path(entity_path: str) -> None:
+    attach(set_value("entity_path", entity_path))
 
 
-def get_chained_entity_name(entity_name: str) -> str:
-    parent = get_value("entity_name")
+def get_chained_entity_path(entity_name: str) -> str:
+    parent = get_value("entity_path")
     if parent is None:
         return entity_name
     else:
@@ -509,35 +338,134 @@ def init_tracer_provider(resource: Resource) -> TracerProvider:
     return provider
 
 
-def init_instrumentations(should_enrich_metrics: bool):
-    init_openai_instrumentor(should_enrich_metrics)
-    init_anthropic_instrumentor(should_enrich_metrics)
-    init_cohere_instrumentor()
-    init_pinecone_instrumentor()
-    init_qdrant_instrumentor()
-    init_chroma_instrumentor()
-    init_google_generativeai_instrumentor()
-    init_haystack_instrumentor()
-    init_langchain_instrumentor()
-    init_mistralai_instrumentor()
-    init_ollama_instrumentor()
-    init_llama_index_instrumentor()
-    init_milvus_instrumentor()
-    init_transformers_instrumentor()
-    init_together_instrumentor()
-    init_requests_instrumentor()
-    init_urllib3_instrumentor()
-    init_pymysql_instrumentor()
-    init_bedrock_instrumentor(should_enrich_metrics)
-    init_replicate_instrumentor()
-    init_vertexai_instrumentor()
-    init_watsonx_instrumentor()
-    init_weaviate_instrumentor()
-    init_alephalpha_instrumentor()
-    init_marqo_instrumentor()
+def init_instrumentations(
+    should_enrich_metrics: bool,
+    base64_image_uploader: Callable[[str, str, str], str],
+    instruments: Optional[Set[Instruments]] = None,
+    block_instruments: Optional[Set[Instruments]] = None,
+):
+    block_instruments = block_instruments or set()
+    instruments = instruments or set(
+        Instruments
+    )  # Use all instruments if none specified
+
+    # Remove any instruments that were explicitly blocked
+    instruments = instruments - block_instruments
+
+    instrument_set = False
+    for instrument in instruments:
+        if instrument == Instruments.ALEPHALPHA:
+            if init_alephalpha_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.ANTHROPIC:
+            if init_anthropic_instrumentor(
+                should_enrich_metrics, base64_image_uploader
+            ):
+                instrument_set = True
+        elif instrument == Instruments.BEDROCK:
+            if init_bedrock_instrumentor(should_enrich_metrics):
+                instrument_set = True
+        elif instrument == Instruments.CHROMA:
+            if init_chroma_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.COHERE:
+            if init_cohere_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.GOOGLE_GENERATIVEAI:
+            if init_google_generativeai_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.GROQ:
+            if init_groq_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.HAYSTACK:
+            if init_haystack_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.LANCEDB:
+            if init_lancedb_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.LANGCHAIN:
+            if init_langchain_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.LLAMA_INDEX:
+            if init_llama_index_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.MARQO:
+            if init_marqo_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.MILVUS:
+            if init_milvus_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.MISTRAL:
+            if init_mistralai_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.OLLAMA:
+            if init_ollama_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.OPENAI:
+            if init_openai_instrumentor(should_enrich_metrics, base64_image_uploader):
+                instrument_set = True
+        elif instrument == Instruments.PINECONE:
+            if init_pinecone_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.PYMYSQL:
+            if init_pymysql_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.QDRANT:
+            if init_qdrant_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.REDIS:
+            if init_redis_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.REPLICATE:
+            if init_replicate_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.REQUESTS:
+            if init_requests_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.SAGEMAKER:
+            if init_sagemaker_instrumentor(should_enrich_metrics):
+                instrument_set = True
+        elif instrument == Instruments.TOGETHER:
+            if init_together_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.TRANSFORMERS:
+            if init_transformers_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.URLLIB3:
+            if init_urllib3_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.VERTEXAI:
+            if init_vertexai_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.WATSONX:
+            if init_watsonx_instrumentor():
+                instrument_set = True
+        elif instrument == Instruments.WEAVIATE:
+            if init_weaviate_instrumentor():
+                instrument_set = True
+        else:
+            print(Fore.RED + f"Warning: {instrument} instrumentation does not exist.")
+            print(
+                "Usage:\n"
+                "from traceloop.sdk.instruments import Instruments\n"
+                "Traceloop.init(app_name='...', instruments=set([Instruments.OPENAI]))"
+            )
+            print(Fore.RESET)
+
+    if not instrument_set:
+        print(
+            Fore.RED
+            + "Warning: No valid instruments set. "
+            + "Specify instruments or remove 'instruments' argument to use all instruments."
+        )
+        print(Fore.RESET)
+
+    return instrument_set
 
 
-def init_openai_instrumentor(should_enrich_metrics: bool):
+def init_openai_instrumentor(
+    should_enrich_metrics: bool, base64_image_uploader: Callable[[str, str, str], str]
+):
     try:
         if is_package_installed("openai"):
             Telemetry().capture("instrumentation:openai:init")
@@ -548,6 +476,7 @@ def init_openai_instrumentor(should_enrich_metrics: bool):
                 enrich_assistant=should_enrich_metrics,
                 enrich_token_usage=should_enrich_metrics,
                 get_common_metrics_attributes=metrics_common_attributes,
+                upload_base64_image=base64_image_uploader,
             )
             if not instrumentor.is_instrumented_by_opentelemetry:
                 instrumentor.instrument()
@@ -559,7 +488,9 @@ def init_openai_instrumentor(should_enrich_metrics: bool):
         return False
 
 
-def init_anthropic_instrumentor(should_enrich_metrics: bool):
+def init_anthropic_instrumentor(
+    should_enrich_metrics: bool, base64_image_uploader: Callable[[str, str, str], str]
+):
     try:
         if is_package_installed("anthropic"):
             Telemetry().capture("instrumentation:anthropic:init")
@@ -569,6 +500,7 @@ def init_anthropic_instrumentor(should_enrich_metrics: bool):
                 exception_logger=lambda e: Telemetry().log_exception(e),
                 enrich_token_usage=should_enrich_metrics,
                 get_common_metrics_attributes=metrics_common_attributes,
+                upload_base64_image=base64_image_uploader,
             )
             if not instrumentor.is_instrumented_by_opentelemetry:
                 instrumentor.instrument()
@@ -879,6 +811,24 @@ def init_bedrock_instrumentor(should_enrich_metrics: bool):
         return False
 
 
+def init_sagemaker_instrumentor(should_enrich_metrics: bool):
+    try:
+        if is_package_installed("boto3"):
+            from opentelemetry.instrumentation.sagemaker import SageMakerInstrumentor
+
+            instrumentor = SageMakerInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+                enrich_token_usage=should_enrich_metrics,
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing SageMaker instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
 def init_replicate_instrumentor():
     try:
         if is_package_installed("replicate"):
@@ -899,7 +849,7 @@ def init_replicate_instrumentor():
 
 def init_vertexai_instrumentor():
     try:
-        if is_package_installed("vertexai"):
+        if is_package_installed("google-cloud-aiplatform"):
             Telemetry().capture("instrumentation:vertexai:init")
             from opentelemetry.instrumentation.vertexai import VertexAIInstrumentor
 
@@ -917,7 +867,9 @@ def init_vertexai_instrumentor():
 
 def init_watsonx_instrumentor():
     try:
-        if is_package_installed("ibm_watson_machine_learning"):
+        if is_package_installed("ibm-watsonx-ai") or is_package_installed(
+            "ibm-watson-machine-learning"
+        ):
             Telemetry().capture("instrumentation:watsonx:init")
             from opentelemetry.instrumentation.watsonx import WatsonxInstrumentor
 
@@ -987,6 +939,55 @@ def init_marqo_instrumentor():
         return False
 
 
+def init_lancedb_instrumentor():
+    try:
+        if is_package_installed("lancedb"):
+            Telemetry().capture("instrumentation:lancedb:init")
+            from opentelemetry.instrumentation.lancedb import LanceInstrumentor
+
+            instrumentor = LanceInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing LanceDB instrumentor: {e}")
+
+
+def init_redis_instrumentor():
+    try:
+        if is_package_installed("redis"):
+            from opentelemetry.instrumentation.redis import RedisInstrumentor
+
+            instrumentor = RedisInstrumentor()
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument(excluded_urls=EXCLUDED_URLS)
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing redis instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
+def init_groq_instrumentor():
+    try:
+        if is_package_installed("groq"):
+            Telemetry().capture("instrumentation:groq:init")
+            from opentelemetry.instrumentation.groq import GroqInstrumentor
+
+            instrumentor = GroqInstrumentor(
+                exception_logger=lambda e: Telemetry().log_exception(e),
+            )
+            if not instrumentor.is_instrumented_by_opentelemetry:
+                instrumentor.instrument()
+        return True
+    except Exception as e:
+        logging.error(f"Error initializing Groq instrumentor: {e}")
+        Telemetry().log_exception(e)
+        return False
+
+
 def metrics_common_attributes():
     common_attributes = {}
     workflow_name = get_value("workflow_name")
